<?php

namespace Drupal\entitree;

use metalinspired\NestedSet\Config;
use metalinspired\NestedSet\HybridFind;
use metalinspired\NestedSet\HybridManipulate;

class NestedSetFactory implements NestedSetFactoryInterface{
  protected $config;

  protected $find;

  protected $manipulate;

  public function __construct(string $langcode){
    $this->createConfig($langcode);

    $this->find = new HybridFind($this->config);
    $this->manipulate = new HybridManipulate($this->config);
  }

  protected function createConfig(string $langcode)
  {
    $connectionOptions = \Drupal::database()->getConnectionOptions();
    $port = isset($connectionOptions['port']) ? $connectionOptions['port'] : 3306;
    $dsn = $connectionOptions['driver'] . ':dbname=' . $connectionOptions['database'] . ';host=' . $connectionOptions['host'] . ':' . $port;
    $this->config = Config::createWithDsn($dsn, $connectionOptions['username'], $connectionOptions['password']);
    $this->config->table = 'entitree_structure_' . $langcode;
  }

  public function getConfig() : Config
  {
    return $this->config;
  }

  public function getRootNodeId() : int
  {
    return (int)$this->find->getRootNodeId();
  }

  /** MANIPULATE METHODS **/
  public function createRootNode(){
    $this->manipulate->createRootNode();
  }

  public function insert(
    $parent,
    array $data = []
  ){
    return $this->manipulate->insert($parent, $data);
  }

  /** FIND METHODS **/

  public function findDescendants(
    $id,
    $columns = null,
    $order = null,
    Predicate $where = null,
    $depthLimit = null,
    $includeSearchingNode = null
  ){
    $result = $this->find->findDescendants($id, $columns, $order, $where, $depthLimit, $includeSearchingNode);

    $descendants = [];
    while ($row = $result->next()) {
      $descendants[] = $row;
    }
    return $descendants;
  }

  public function findChildren(
    $id,
    $columns = null,
    $order = null,
    Predicate $where = null,
    $includeSearchingNode = null
  ): array{
    $result = $this->find->findChildren($id, $columns, $order, $where, $includeSearchingNode);

    $children = [];
    while($row = $result->next()){
      $children[] = $row;
    }

    return $children;
  }

  public function findAncestors(
    $id,
    $columns = null,
    $order = null,
    Predicate $where = null,
    $depthLimit = null,
    $includeSearchingNode = null
  ): array{
    $result = $this->find->findAncestors($id, $columns, $order, $where, $depthLimit, $includeSearchingNode);

    $ancestors = [];
    while($row = $result->next()){
      $ancestors[] = $row;
    }
    return $ancestors;
  }

  public function findParent($id, $columns = null, $includeSearchingNode = null){
    $result = $this->find->findParent($id, $columns, $includeSearchingNode);
    $parentRow = $result->next();
    return $parentRow;
  }

  /**
   * Wrapper for HybridManipulate::moveMakeChild
   *
   * @param integer $source nested set node ID to move
   * @param integer $destination nested set node ID of destination parent
   * @return void
   */
  public function move(int $source, int $destination){
    $this->manipulate->moveMakeChild($source, $destination);
  }

  /**
   * Wrapper for HybridManipulate::delete
   * Deletes a node and all of its descendants
   *
   * @param integer $node nested set node ID to remove
   * @return void
   */
  public function delete(int $node){
    $this->manipulate->delete($node);
  }
}
