<?php

namespace Drupal\entitree\Plugin\EntitreeEntityType;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\entitree\Plugin\EntitreeEntityTypeBase;
use Symfony\Component\Routing\Route;
use Drupal\Core\Entity\EntityInterface;

/**
 * Provides entitree support for nodes
 *
 * @EntitreeEntityType(
 *   id = "empty_location",
 *   entity_id = "empty_location",
 *   entity_name = "Empty Location"
 * )
 */
class EntitreeEmptyLocation extends EntitreeEntityTypeBase {
  public function buildManageLocationRoute(array &$routes) {
    $route = (new Route('/admin/structure/entitree/{empty_location}/locations/{entity_type}'))
      ->addDefaults([
        '_controller' => '\Drupal\entitree\Controller\EntitreeDisplayController::RenderEntityLocations',
        '_title' => 'Manage Locations',
        'entity_type' => 'empty_location'
      ])
      ->setRequirement('empty_location', '\d+')
      /**
       * @todo setup correct permissions
       */
      //      ->setRequirement('_entity_access', 'node.update')
      ->setRequirement('_permission', 'access content')
      ->setOption('parameters', [
        'emtpy_location' => [
          "type" => "EmptyLocation"
        ]
      ]);
    $routes['entitree.locations.empty_location'] = $route;
  }

  public function getSourcePath($id): string {
    return "/location/$id";
  }

  public function loadEntityByRouteParams(array $params): EntityInterface {
    return \Drupal\entitree\Entity\EmptyLocation::load($params['empty_location']);
  }

  public function getLocationsPathById(int $id): \Drupal\Core\Url{
    return \Drupal\Core\Url::fromRoute('entitree.locations.empty_location', [
      "empty_location" => $id
    ]);
  }

  public function getLocalTasks($base_plugin_definition): array {
    $tasks = [];

    return $tasks;
  }

  /**
   * {@inheritDoc}
   */
  public static function getExistingPathAliases(int $id = NULL): array {
    if ($id) {
      $query = "SELECT * FROM {path_alias} WHERE path = '/location/$id'";
      $result = \Drupal::database()->query($query);
    }
    else {
      $query = "SELECT * FROM {path_alias} WHERE path LIKE '/location/%'";
      $result = \Drupal::database()->query($query);
    }

    return $result->fetchAllAssoc('id');
  }

  /**
   * {@inheritDoc}
   */
  public static function getOrphanedpathAliases(int $id): array {
    $aliases = self::getExistingPathAliases($id);

    // Determine which of these have locations for this empty location.
    /** @var \Drupal\entitree\EntitreeManagerInterface $entitreeManager */
    $entitreeManager = \Drupal::service('entitree.manager');
    $locations = $entitreeManager->getLocationsByIdAndType($id, 'empty_location');

    $orphaned = [];
    foreach ($aliases as $alias) {
      // Search locations for one associated with this path alias ID.
      foreach ($locations as $location) {
        if ($location->get('pid')->getString() == $alias['id']) {
          continue 2;
        }
      }

      // If reaching this point the alias does not have a corresponding location.
      $orphaned[] = $alias;
    }

    return $orphaned;
  }

}
