<?php

namespace Drupal\entitree\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Entity\EntityInterface;

/**
 * Defines an interface for Entitree entity type plugins.
 */
interface EntitreeEntityTypeInterface extends PluginInspectionInterface {
  /**
   * Defines the route to manage Entitree locations for this entity type.
   *
   * @param array $routes
   */
  public function buildManageLocationRoute(array &$routes);

  /**
   * Apply token replace action to a string using values from the provided entity.
   *
   * @param string $value
   * @param EntityInterface $entity
   * @return string
   */
  public static function applyTokenReplaceActions(string $value, $entity): string;

  /**
   * Defines the source path to use when generating URL Aliases.
   *
   * @param int|string $id
   * @return string
   */
  public function getSourcePath($id): string;

  public function loadEntityByRouteParams(array $params): EntityInterface;
  public function getLocationsPathById(int $entity_id): \Drupal\Core\Url;
  public function getLocalTasks($base_plugin_definition): array;

  /**
   * Provide a list of default operations that may be used by type plugins as needed.
   *
   * @return array
   */
  public function getOperations(): array;

  /**
   * Provide an array of token types available for this entity type.
   *
   * @return array
   */
  public static function getTokenTypes(): array;

  /**
   * Load all path aliases for this Entity type.
   *
   * @param int $id
   *   ID of the entity to restrict the search to paths related.
   *
   * @return array
   *   Array of rows from the path_alias table.
   */
  public static function getExistingPathAliases(int $id = NULL): array;

  /**
   * Get all path aliases that do not correspond to an Entitree location.
   *
   * @param int $id
   *   Entity ID associated with the path aliases.
   *
   * @return array
   *   Array of rows from the path_alias table.
   */
  public static function getOrphanedpathAliases(int $id): array;

}
